@@ -1,6 +1,6 @@
-from zeep.client import Client  # noqa
+from zeep.client import CachingClient, Client  # noqa
 from zeep.transports import Transport  # noqa
 from zeep.plugins import Plugin  # noqa
 from zeep.xsd.valueobjects import AnyObject  # noqa
 
-__version__ = '1.1.0'
+__version__ = '1.2.0'
