@@ -4,7 +4,7 @@ from collections import OrderedDict, defaultdict, deque
 
 from cached_property import threaded_cached_property
 
-from zeep.exceptions import UnexpectedElementError
+from zeep.exceptions import UnexpectedElementError, ValidationError
 from zeep.xsd.const import NotSet, SkipValue
 from zeep.xsd.elements import Any, Element
 from zeep.xsd.elements.base import Base
@@ -115,7 +115,10 @@ class OrderIndicator(Indicator, list):
         if self.accepts_multiple:
             assert name
 
-        if name and name in available_kwargs:
+        if name:
+            if name not in available_kwargs:
+                return {}
+
             assert self.accepts_multiple
 
             # Make sure we have a list, lame lame
@@ -125,7 +128,12 @@ class OrderIndicator(Indicator, list):
 
             result = []
             for item_value in max_occurs_iter(self.max_occurs, item_kwargs):
-                item_kwargs = set(item_value.keys())
+                try:
+                    item_kwargs = set(item_value.keys())
+                except AttributeError:
+                    raise TypeError(
+                        "A list of dicts is expected for unbounded Sequences")
+
                 subresult = OrderedDict()
                 for item_name, element in self.elements:
                     value = element.parse_kwargs(item_value, item_name, item_kwargs)
@@ -148,7 +156,6 @@ class OrderIndicator(Indicator, list):
             return result
 
         else:
-            assert not name
             assert not self.accepts_multiple
             result = OrderedDict()
             for elm_name, element in self.elements_nested:
@@ -170,6 +177,8 @@ class OrderIndicator(Indicator, list):
         else:
             values = value
 
+        self.validate(values, render_path)
+
         for value in max_occurs_iter(self.max_occurs, values):
             for name, element in self.elements_nested:
                 if name:
@@ -189,11 +198,16 @@ class OrderIndicator(Indicator, list):
                 if element_value is not None or not element.is_optional:
                     element.render(parent, element_value, child_path)
 
+    def validate(self, value, render_path):
+        for item in value:
+            if item is NotSet:
+                raise ValidationError("No value set", path=render_path)
+
     def signature(self, schema=None, standalone=True):
         parts = []
         for name, element in self.elements_nested:
             if isinstance(element,  Indicator):
-                parts.append(element.signature(schema))
+                parts.append(element.signature(schema, standalone=False))
             else:
                 value = element.signature(schema, standalone=False)
                 parts.append('%s: %s' % (name, value))
@@ -459,15 +473,33 @@ class Sequence(OrderIndicator):
 
     """
     def parse_xmlelements(self, xmlelements, schema, name=None, context=None):
+        """
+        :param xmlelements: Dequeue of XML element objects
+        :type xmlelements: collections.deque of lxml.etree._Element
+        :param schema: The parent XML schema
+        :type schema: zeep.xsd.Schema
+        :param name: The name of the parent element
+        :type name: str
+        :param context: Optional parsing context (for inline schemas)
+        :type context: zeep.xsd.context.XmlParserContext
+        :return: dict or None
+
+        """
         result = []
+
         for _unused in max_occurs_iter(self.max_occurs):
             if not xmlelements:
                 break
 
             item_result = OrderedDict()
             for elm_name, element in self.elements:
-                item_subresult = element.parse_xmlelements(
-                    xmlelements, schema, name, context=context)
+                try:
+                    item_subresult = element.parse_xmlelements(
+                        xmlelements, schema, name, context=context)
+                except UnexpectedElementError:
+                    if schema.strict:
+                        raise
+                    item_subresult = None
 
                 # Unwrap if allowed
                 if isinstance(element, OrderIndicator):
@@ -513,6 +545,14 @@ class Group(Indicator):
             return [('_value_1', self.child)]
         return self.child.elements
 
+    def accept(self, values):
+        """Return the number of values which are accepted by this choice.
+
+        If not all required elements are available then 0 is returned.
+
+        """
+        return self.child.accept(values)
+
     def parse_args(self, args, index=0):
         return self.child.parse_args(args, index)
 
